var CodeMirror = (function() {
  function Event(orig) {this.e = orig;}
  Event.prototype = {
    stop: function() {
      if (this.e.stopPropagation) this.e.stopPropagation();
      else this.e.cancelBubble = true;
      if (this.e.preventDefault) this.e.preventDefault();
      else this.e.returnValue = false;
    },
    target: function() {
      return this.e.target || this.e.srcElement;
    },
    button: function() {
      if (this.e.which) return this.e.which;
      else if (this.e.button & 1) return 1;
      else if (this.e.button & 2) return 3;
      else if (this.e.button & 4) return 2;
    },
    pageX: function() {
      if (this.e.pageX != null) return this.e.pageX;
      else return this.e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
    },
    pageY: function() {
      if (this.e.pageY != null) return this.e.pageY;
      else return this.e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
    }
  };

  function connect(node, type, handler, disconnect) {
    function wrapHandler(event) {handler(new Event(event || window.event));}
    if (typeof node.addEventListener == "function") {
      node.addEventListener(type, wrapHandler, false);
      if (disconnect) return function() {node.removeEventListener(type, wrapHandler, false);};
    }
    else {
      node.attachEvent("on" + type, wrapHandler);
      if (disconnect) return function() {node.detachEvent("on" + type, wrapHandler);};
    }
  }

  var badInnerHTML = (function() {
    var pre = document.createElement("pre");
    pre.innerHTML = " "; return !pre.innerHTML;
  })();
  // TODO can this reliably be feature-detected?
  var lineSep = (window.opera || /\bMSIE\b/.test(navigator.userAgent)) ? "\r\n" : "\n";
  var tabSize = 8;

  function countIndentation(string) {
    var spaceEnd = string.search(/[^\s\u00a0]/), n = 0;
    for (var i = 0; i < spaceEnd; ++i) {
      if (string.charAt(i) == "\t") n += tabSize - (n % tabSize);
      else ++n;
    }
    return n;
  }

  function eltOffset(node) {
    var x = 0, y = 0, n2 = node;
    for (var n = node; n; n = n.offsetParent) {x += n.offsetLeft; y += n.offsetTop;}
    for (var n = node; n != document.body; n = n.parentNode) {x -= n.scrollLeft; y -= n.scrollTop;}
    return {left: x, top: y};
  }

  function posEq(a, b) {return a.line == b.line && a.ch == b.ch;}
  function posLess(a, b) {return a.line < b.line || (a.line == b.line && a.ch < b.ch);}
  function copyPos(x) {return {line: x.line, ch: x.ch};}

  function htmlEscape(str) {
    return str.replace(/[<&]/g, function(str) {return str == "&" ? "&amp;" : "&lt;";});
  }

  function editEnd(from, to) {
    if (!to) return from ? from.length : 0;
    if (!from) return to.length;
    for (var i = from.length, j = to.length; i >= 0 && j >= 0; --i, --j)
      if (from.charAt(i) != to.charAt(j)) break;
    return j + 1;
  }

  function indexOf(collection, elt) {
    if (collection.indexOf) return collection.indexOf(elt);
    for (var i = 0, e = collection.length; i < e; ++i)
      if (collection[i] == elt) return i;
    return -1;
  }
  if ("\n\nb".split(/\n/).length != 3) // Various broken IE split implementations
    var splitLines = function(string) {
      var pos = 0, nl, result = [];
      while ((nl = string.indexOf("\n", pos)) > -1) {
        result.push(string.slice(pos, string.charAt(nl-1) == "\r" ? nl - 1 : nl));
        pos = nl + 1;
      }
      result.push(string.slice(pos));
      return result;
    };
  else
    var splitLines = function(string){return string.split(/\r?\n/);};

  if (window.getSelection) {
    var selRange = function(te) {
      return {start: te.selectionStart, end: te.selectionEnd};
    };
    var setSelRange = function(te, start, end) {
      te.setSelectionRange(start, end);
    };
  }
  else {
    var selRange = function(te) {
      try {var range = document.selection.createRange();}
      catch(e) {return null;}
      if (!range || range.parentElement() != te) return null;
      var val = te.value, len = val.length, localRange = te.createTextRange();
      localRange.moveToBookmark(range.getBookmark());
      var endRange = te.createTextRange();
      endRange.collapse(false);

      if (localRange.compareEndPoints("StartToEnd", endRange) > -1)
        return {start: len, end: len};

      var start = -localRange.moveStart("character", -len);
      for (var i = val.indexOf("\r"); i > -1 && i < start; i = val.indexOf("\r", i+1), start++);

      if (localRange.compareEndPoints("EndToEnd", endRange) > -1)
        return {start: start, end: len};

      var end = -localRange.moveEnd("character", -len);
      for (var i = val.indexOf("\r"); i > -1 && i < end; i = val.indexOf("\r", i+1), end++);
      return {start: start, end: end};
    };
    var setSelRange = function(te, start, end) {
      var range = te.createTextRange();
      range.collapse(true);
      var endrange = range.duplicate();
      var newlines = 0, txt = te.value;
      for (var pos = txt.indexOf("\n"); pos > -1 && pos < start; pos = txt.indexOf("\n", pos + 1))
        ++newlines;
      range.move("character", start - newlines);
      for (; pos > -1 && pos < end; pos = txt.indexOf("\n", pos + 1))
        ++newlines;
      endrange.move("character", end - newlines);
      range.setEndPoint("EndToEnd", endrange);
      range.select();
    };
  }

  var movementKeys = {};
  for (var i = 35; i <= 40; ++i)
    movementKeys[i] = movementKeys["c" + i] = true;

  function Delayed() {this.id = null;}
  Delayed.prototype = {set: function(ms, f) {clearTimeout(this.id); this.id = setTimeout(f, ms);}};

  function copyStyles(from, to, source, dest) {
    for (var i = 0, pos = 0, state = 0; pos < to; i+=2) {
      var part = source[i], end = pos + part.length;
      if (state === 0) {
        if (end > from) dest.push(part.slice(from - pos, Math.min(part.length, to - pos)), source[i+1]);
        if (end >= from) state = 1;
      }
      else if (state === 1) {
        if (end > to) dest.push(part.slice(0, to - pos), source[i+1]);
        else dest.push(part, source[i+1]);
      }
      pos = end;
    }
  }

  function Line(text, styles) {
    this.styles = styles || [text, null];
    this.stateAfter = null;
    this.text = text;
    this.marked = null; this.gutterMarker = null;
  }
  Line.prototype = {
    replace: function(from, to, text) {
      var st = [], mk = this.marked;
      copyStyles(0, from, this.styles, st);
      if (text) st.push(text, null);
      copyStyles(to, this.text.length, this.styles, st);
      this.styles = st;
      this.text = this.text.slice(0, from) + text + this.text.slice(to);
      this.stateAfter = null;
      if (mk) {
        var diff = text.length - (to - from), end = this.text.length;
        function fix(n) {return n <= Math.min(to, to + diff) ? n : n + diff;}
        for (var i = 0; i < mk.length; ++i) {
          var mark = mk[i], del = false;
          if (mark.from >= end) del = true;
          else {mark.from = fix(mark.from); if (mark.to != null) mark.to = fix(mark.to);}
          if (del || mark.from >= mark.to) {mk.splice(i, 1); i--;}
        }
      }
    },
    split: function(pos, textBefore) {
      var st = [textBefore, null];
      copyStyles(pos, this.text.length, this.styles, st);
      return new Line(textBefore + this.text.slice(pos), st);
    },
    addMark: function(from, to, style) {
      var mk = this.marked, mark = {from: from, to: to, style: style};
      if (this.marked == null) this.marked = [];
      this.marked.push(mark);
      this.marked.sort(function(a, b){return a.from - b.from;});
      return mark;
    },
    removeMark: function(mark) {
      var mk = this.marked;
      if (!mk) return;
      for (var i = 0; i < mk.length; ++i)
        if (mk[i] == mark) {mk.splice(i, 1); break;}
    },
    highlight: function(mode, state) {
      var stream = new StringStream(this.text), st = this.styles;
      st.length = 0;
      while (!stream.eol()) {
        var style = mode.token(stream, state);
        var substr = this.text.slice(stream.start, stream.pos);
        stream.start = stream.pos;
        if (st.length && st[st.length-1] == style)
          st[st.length-2] += substr;
        else if (substr)
          this.styles.push(substr, style);
        // Give up when line is ridiculously long
        if (stream.pos > 5000) {
          this.styles.push(this.text.slice(stream.pos), null);
          break;
        }
      }
    },
    indentation: function() {return countIndentation(this.text);},
    getHTML: function(sfrom, sto, includePre) {
      var html = includePre ? ["<pre>"] : [];
      function span(text, style) {
        if (!text) return;
        if (style) html.push('<span class="', style, '">', htmlEscape(text), "</span>");
        else html.push(htmlEscape(text));
      }
      function finish() {
        if (includePre) html.push("</pre>");
        return html.join("");
      }
      if (!this.marked && sfrom === 0 && sto === null) {
        span(this.text + " ", "CodeMirror-selected");
        return finish();
      }

      var st = this.styles, allText = this.text, marked = this.marked;
      if (sfrom === sto) sfrom = null;

      if (!allText)
        span(" ");
      else if (!marked && sfrom === null)
        for (var i = 0, e = st.length; i < e; i+=2) span(st[i], st[i+1]);
      else {
        var pos = 0, i = 0, text = "", style, sg = 0;
        function copyUntil(end) {
          for (;;) {
            var upto = pos + text.length;
            span(upto > end ? text.slice(0, end - pos) : text, style);
            if (upto >= end) {text = text.slice(end - pos); pos = end; break;}
            pos = upto;
            text = st[i++]; style = st[i++];
          }
        }
        function chunkUntil(end, cStyle) {
          var acc = [];
          for (;;) {
            var upto = pos + text.length;
            if (upto >= end) {
              var size = end - pos;
              acc.push(text.slice(0, size));
              span(acc.join(""), cStyle);
              text = text.slice(size);
              pos += size;
              break;
            }
            acc.push(text);
            pos = upto;
            text = st[i++]; style = st[i++];
          }
        }
        var markpos = 0, mark;
        function nextMark() {
          if (!marked) return null;
          for (; markpos < marked.length; markpos++) {
            mark = marked[markpos];
            var end = mark.to == null ? allText.length : mark.to;
            if (end > pos) return Math.max(mark.from, pos);
          }
        }

        while (pos < allText.length) {
          var nextmark = nextMark();
          if (sfrom != null && sfrom >= pos && (nextmark == null || sfrom <= nextmark)) {
            copyUntil(sfrom);
            if (sto == null) {
              span(allText.slice(pos) + " ", "CodeMirror-selected");
              break;
            }
            chunkUntil(sto, "CodeMirror-selected");
          }
          else if (nextmark != null) {
            copyUntil(nextmark);
            var end = mark.to == null ? allText.length : mark.to;
            chunkUntil(sfrom == null || sfrom < pos ? end : Math.min(sfrom, end), mark.style);
          }
          else copyUntil(allText.length);
        }
      }
      return finish();
    }
  };

  function History() {
    this.time = 0;
    this.done = []; this.undone = [];
  }
  History.prototype = {
    addChange: function(start, added, old) {
      this.undone.length = 0;
      var time = +new Date, last = this.done[this.done.length - 1];
      if (time - this.time > 400 || !last ||
          last.start > start + added || last.start + last.added < start - last.added + last.old.length)
        this.done.push({start: start, added: added, old: old});
      else {
        var oldoff = 0;
        if (start < last.start) {
          for (var i = last.start - start - 1; i >= 0; --i)
            last.old.unshift(old[i]);
          last.added += last.start - start;
          last.start = start;
        }
        else if (last.start < start) {
          oldoff = start - last.start;
          added += oldoff;
        }
        for (var i = last.added, e = old.length; i < e; ++i)
          last.old.push(old[i]);
        if (last.added < added) last.added = added;
      }
      this.time = time;
    }
  };

  function CodeMirror(place, options) {
    if (!options) options = {};
    var defaults = CodeMirror.defaults;
    for (var opt in defaults)
      if (defaults.hasOwnProperty(opt) && !options.hasOwnProperty(opt))
        options[opt] = defaults[opt];

    var div = document.createElement("div");
    div.className = "CodeMirror";
    div.innerHTML =
      '<div style="position: relative">' +
        '<pre style="position: absolute; visibility: hidden"><span>-</span></pre>' +
        '<div style="position: relative">' +
          '<div class="CodeMirror-gutter"></div>' +
          '<div style="overflow: hidden; position: absolute; width: 0">' +
            '<textarea style="position: absolute; width: 10000px;"></textarea></div>' +
          '<div class="CodeMirror-lines"><div style="position: relative">' +
            '<pre class="CodeMirror-cursor">&nbsp;</pre>' +
            '<div></div></div></div></div></div>';
    if (place.appendChild) place.appendChild(div); else place(div);
    var code = div.firstChild, measure = code.firstChild, mover = measure.nextSibling,
        gutter = mover.firstChild, inputDiv = gutter.nextSibling, input = inputDiv.firstChild,
        lineWrap = inputDiv.nextSibling.firstChild, cursor = lineWrap.firstChild, lineDiv = cursor.nextSibling;
//    inputDiv.style.width = "30em"; inputDiv.style.height = "5em"; input.rows = 3;
    if (options.tabindex != null) input.tabindex = options.tabindex;
    if (options.lineNumbers) {options.gutter = "ln";}
    if (!options.gutter) gutter.style.display = "none";

    var poll = new Delayed(), highlight = new Delayed(), blinker;

    var mode, lines = [new Line("")], work, history = new History();
    setMode(options.mode);
    var zero = {line: 0, ch: 0};
    var sel = {from: zero, to: zero, inverted: false};
    var shiftSelecting, reducedSelection, focused, updateInput, changes, textChanged, selectionChanged;
    var showingFrom = 0, showingTo = 0, editing, bracketHighlighted;
    operation(function(){setValue(options.value || ""); updateInput = false;})();
    setTimeout(prepareInput, 20);

    connect(code, "mousedown", operation(onMouseDown));
    connect(code, "dblclick", operation(onDblClick));
    connect(div, "scroll", updateDisplay);
    connect(window, "resize", updateDisplay);
    connect(input, "keyup", operation(onKeyUp));
    connect(input, "keydown", operation(onKeyDown));
    connect(input, "keypress", operation(onKeyPress));
    connect(input, "focus", onFocus);
    connect(input, "blur", onBlur);

    connect(div, "dragenter", function(e){e.stop();});
    connect(div, "dragover", function(e){e.stop();});
    connect(div, "drop", operation(onDrop));
    connect(div, "paste", function(){input.focus(); fastPoll();});
    connect(input, "paste", fastPoll);

    function stopScroll(e) {
      var up = e.e.detail ? e.e.detail < 0 : e.e.wheelDelta > 0;
      if (up ? div.scrollTop == 0 : div.scrollTop + div.clientHeight >= div.scrollHeight)
        e.stop();
    }
    connect(div, "mousewheel", stopScroll);
    connect(div, "DOMMouseScroll", stopScroll);

    if (document.activeElement == input) onFocus();
    else onBlur();

    function setValue(code) {
      history = null;
      var top = {line: 0, ch: 0};
      updateLines(top, {line: lines.length - 1, ch: lines[lines.length-1].text.length},
                  splitLines(code), top, top);
      history = new History();
    }
    function getValue(code) {
      var text = [];
      for (var i = 0, l = lines.length; i < l; ++i)
        text.push(lines[i].text);
      return text.join("\n");
    }

    function onMouseDown(e) {
      var start = posFromMouse(e), last = start, target = e.target();
      if (!start) return;
      setCursor(start.line, start.ch, false);
      if (e.button() != 1) return;
      if (target.parentNode == gutter) {
        if (options.onGutterClick)
          options.onGutterClick(instance, indexOf(gutter.childNodes, target) + showingFrom);
        return;
      }

      if (!focused) onFocus();
      e.stop();
      function end() {
        input.focus();
        updateInput = true;
        move(); up();
      }

      var move = connect(document, "mousemove", operation(function(e) {
        e.stop();
        if (!e.button()) {end(); return;}
        var cur = posFromMouse(e);
        if (cur && !posEq(cur, last)) {
          if (!focused) onFocus();
          last = cur;
          setSelection(start, cur);
          updateInput = false;
        }
      }), true);
      var up = connect(document, "mouseup", operation(function(e) {
        var cur = posFromMouse(e);
        if (cur) setSelection(start, cur);
        e.stop();
        end();
      }), true);
    }
    function onDblClick(e) {
      var pos = posFromMouse(e);
      if (!pos) return;
      selectWordAt(pos);
      e.stop();
    }
    function onDrop(e) {
      var pos = posFromMouse(e), files = e.e.dataTransfer.files;
      if (!pos || options.readOnly) return;
      if (files && files.length && window.FileReader && window.File) {
        var n = files.length, text = Array(n), read = 0;
        for (var i = 0; i < n; ++i) loadFile(files[i], i);
        function loadFile(file, i) {
          var reader = new FileReader;
          reader.onload = function() {
            text[i] = reader.result;
            if (++read == n) replaceRange(text.join(""), clipPos(pos), clipPos(pos));
          };
          reader.readAsText(file);
        }
      }
      else {
        try {
          var text = e.e.dataTransfer.getData("Text");
          if (text) replaceRange(text, pos, pos);
        }
        catch(e){}
      }
    }
    function onKeyDown(e) {
      if (!focused) onFocus();

      var code = e.e.keyCode, ctrl = e.e.ctrlKey && !e.e.altKey, done = false;
      if (e.e.shiftKey) shiftSelecting = shiftSelecting || (sel.inverted ? sel.to : sel.from);
      else shiftSelecting = null;

      if (code == 33 || code == 34) {scrollPage(code == 34); done = true;} // page up/down
      else if (ctrl && (code == 36 || code == 35)) {scrollEnd(code == 36); done = true;} // ctrl-home/end
      else if (ctrl && code == 65) {selectAll(); done = true;} // ctrl-a
      else if (!options.readOnly) {
        if (!ctrl && code == 13) {handleEnter(); done = true;} // enter
        else if (!ctrl && code == 9) {if (handleTab(e.e.shiftKey)) done = true;} // tab
        else if (ctrl && code == 90) {undo(); done = true;} // ctrl-z
        else if (ctrl && ((e.e.shiftKey && code == 90) || code == 89)) {redo(); done = true;} // ctrl-shift-z, ctrl-y
      }

      if (done) {e.stop(); return;}
      var id = (ctrl ? "c" : "") + code;
      if (sel.inverted && movementKeys.hasOwnProperty(id)) {
        var range = selRange(input);
        if (range) {
          reducedSelection = {anchor: range.start};
          setSelRange(input, range.start, range.start);
        }
      }
      fastPoll(20, id);
    }
    function onKeyUp(e) {
      if (reducedSelection) {
        reducedSelection = null;
        updateInput = true;
      }
      if (e.e.keyCode == 16) shiftSelecting = null;
    }
    function onKeyPress(e) {
      var code = e.e.keyCode;
      if (code == 13 || (code == 9 && options.tabMode != "default")) e.stop();
    }

    function onFocus() {
      focused = true;
      slowPoll();
      if (div.className.search(/\bCodeMirror-focused\b/) == -1)
        div.className += " CodeMirror-focused";
      restartBlink();
    }
    function onBlur() {
      clearInterval(blinker);
      shiftSelecting = null;
      focused = false;
      div.className = div.className.replace(" CodeMirror-focused", "");
    }

    function updateLines(from, to, newText, selFrom, selTo) {
      if (history) {
        var old = [];
        for (var i = from.line, e = to.line + 1; i < e; ++i) old.push(lines[i].text);
        history.addChange(from.line, newText.length, old);
        while (history.done.length > options.undoDepth) history.done.shift();
      }
      updateLines1(from, to, newText, selFrom, selTo);
    }
    function unredoHelper(from, to) {
      var change = from.pop();
      if (change) {
        var replaced = [], end = change.start + change.added;
        for (var i = change.start; i < end; ++i) replaced.push(lines[i].text);
        to.push({start: change.start, added: change.old.length, old: replaced});
        var pos = clipPos({line: change.start + change.old.length - 1,
                           ch: editEnd(replaced[replaced.length-1], change.old[change.old.length-1])});
        updateLines1({line: change.start, ch: 0}, {line: end - 1, ch: lines[end-1].text.length}, change.old, pos, pos);
      }
    }
    function undo() {unredoHelper(history.done, history.undone);}
    function redo() {unredoHelper(history.undone, history.done);}

    function updateLines1(from, to, newText, selFrom, selTo) {
      // Update lines length and the associated DIVs
      var nlines = to.line - from.line, firstLine = lines[from.line], lastLine = lines[to.line];
      if (from.line == to.line) {
        if (newText.length == 1)
          firstLine.replace(from.ch, to.ch, newText[0]);
        else {
          var lastLine = firstLine.split(to.ch, newText[newText.length-1]);
          var spliceargs = [from.line + 1, nlines];
          firstLine.replace(from.ch, firstLine.text.length, newText[0]);
          for (var i = 1, e = newText.length - 1; i < e; ++i) spliceargs.push(new Line(newText[i]));
          spliceargs.push(lastLine);
          lines.splice.apply(lines, spliceargs);
        }
      }
      else if (newText.length == 1) {
        firstLine.replace(from.ch, firstLine.text.length, newText[0] + lastLine.text.slice(to.ch));
        lines.splice(from.line + 1, nlines);
      }
      else {
        var spliceargs = [from.line + 1, nlines - 1];
        firstLine.replace(from.ch, firstLine.text.length, newText[0]);
        lastLine.replace(0, to.ch, newText[newText.length-1]);
        for (var i = 1, e = newText.length - 1; i < e; ++i) spliceargs.push(new Line(newText[i]));
        lines.splice.apply(lines, spliceargs);
      }

      var newWork = [], lendiff = newText.length - nlines - 1;
      for (var i = 0, l = work.length; i < l; ++i) {
        var task = work[i];
        if (task < from.line) newWork.push(task);
        else if (task > to.line) newWork.push(task + lendiff);
      }
      if (newText.length) newWork.push(from.line);
      work = newWork;
      startWorker(100);
      changes.push({from: from.line, to: to.line + 1, diff: lendiff});
      textChanged = true;

      function updateLine(n) {return n <= Math.min(to.line, to.line + lendiff) ? n : n + lendiff;}
      setSelection(selFrom, selTo, updateLine(sel.from.line), updateLine(sel.to.line));

      code.style.height = (lines.length * lineHeight() + 2 * paddingTop()) + "px";
    }

    function slowPoll() {
      poll.set(2000, function() {
        startOperation();
        readInput();
        if (focused) slowPoll();
        endOperation();
      });
    }
    function fastPoll(keyId) {
      var misses = 0;
      function p() {
        startOperation();
        var state = readInput();
        if (state == "moved" && keyId) movementKeys[keyId] = true;
        if (state) {poll.set(80, p); misses = 0;}
        else if (misses++ < 4) {poll.set(80, p);}
        else slowPoll();
        endOperation();
      }
      poll.set(20, p);
    }

    function readInput() {
      var changed = false, text = input.value, sr = selRange(input);
      if (!sr) return false;
      var changed = editing.text != text, rs = reducedSelection;
      var moved = changed || sr.start != editing.start || sr.end != (rs ? editing.start : editing.end);
      if (reducedSelection && !moved && sel.from.line == 0 && sel.from.ch == 0)
        reducedSelection = null;
      else if (!moved) return false;
      if (changed) {
        shiftSelecting = reducedSelection = null;
        if (options.readOnly) {updateInput = true; return "changed";}
      }

      function computeOffset(n, startLine) {
        var pos = 0;
        while (true) {
          var found = text.indexOf("\n", pos);
          if (found == -1 || (text.charAt(found-1) == "\r" ? found - 1 : found) >= n)
            return {line: startLine, ch: n - pos};
          ++startLine;
          pos = found + 1;
        }
      }
      var from = computeOffset(sr.start, editing.from),
          to = computeOffset(sr.end, editing.from);
      if (rs) {
        from = sr.start == rs.anchor ? to : from;
        to = shiftSelecting ? sel.to : sr.start == rs.anchor ? from : to;
        if (!posLess(from, to)) {
          reducedSelection = null;
          sel.inverted = false;
          var tmp = from; from = to; to = tmp;
        }
      }

      if (from.line == to.line && from.line == sel.from.line && from.line == sel.to.line && !shiftSelecting)
        updateInput = false;
      if (changed) {
        // TODO clean this up. it's awful. seriously
        var start = 0, end = text.length, len = Math.min(end, editing.text.length);
        var c, line = editing.from, nl = -1;
        while (start < len && (c = text.charAt(start)) == editing.text.charAt(start)) {
          ++start;
          if (c == "\n") {line++; nl = start;}
        }
        var ch = nl > -1 ? start - nl : start, endline = editing.to - 1, edend = editing.text.length;
        for (;;) {
          c = editing.text.charAt(edend);
          if (c == "\n") endline--;
          if (text.charAt(end) != c) {++end; ++edend; break;}
          if (edend <= start || end <= start) break;
          --end; --edend;
        }
        var nl = editing.text.lastIndexOf("\n", edend - 1), endch = nl == -1 ? edend : edend - nl - 1;
        updateLines({line: line, ch: ch}, {line: endline, ch: endch}, splitLines(text.slice(start, end)), from, to);
        if (line != endline || from.line != line) updateInput = true;
      }
      else setSelection(from, to);

      editing.text = text; editing.start = sr.start; editing.end = sr.end;
      return changed ? "changed" : moved ? "moved" : false;
    }

    function prepareInput() {
      var text = [];
      var from = Math.max(0, sel.from.line - 1), to = Math.min(lines.length, sel.to.line + 2);
      for (var i = from; i < to; ++i) text.push(lines[i].text);
      text = input.value = text.join(lineSep);
      var startch = sel.from.ch, endch = sel.to.ch;
      for (var i = from; i < sel.from.line; ++i)
        startch += lineSep.length + lines[i].text.length;
      for (var i = from; i < sel.to.line; ++i)
        endch += lineSep.length + lines[i].text.length;
      editing = {text: text, from: from, to: to, start: startch, end: endch};
      setSelRange(input, startch, reducedSelection ? startch : endch);
    }

    function scrollCursorIntoView() {
      var cursor = localCursorCoords(sel.inverted);
      cursor.x += paddingLeft(); cursor.y += paddingTop();
      var screen = div.clientHeight, screentop = div.scrollTop;
      if (cursor.y < screentop)
        div.scrollTop = Math.max(0, cursor.y - 10);
      else if ((cursor.y += lineHeight()) > screentop + screen)
        div.scrollTop = (cursor.y + 10) - screen;

      var screenw = div.offsetWidth, screenleft = div.scrollLeft;
      if (cursor.x < screenleft)
        div.scrollLeft = Math.max(0, cursor.x - 10);
      else if (cursor.x > screenw + screenleft)
        div.scrollLeft = (cursor.x + 10) - screenw;
    }

    function updateDisplay(changes) {
      if (!div.clientWidth) return;
      var lh = lineHeight(), top = div.scrollTop - paddingTop();
      var visibleFrom = Math.max(0, Math.floor(top / lh));
      var visibleTo = Math.min(lines.length, Math.ceil((top + div.clientHeight) / lh));

      var intact = [{from: showingFrom, to: showingTo, domStart: 0}];
      for (var i = 0, l = changes ? changes.length : 0; i < l; ++i) {
        var change = changes[i], intact2 = [], diff = change.diff || 0;
        for (var j = 0, l2 = intact.length; j < l2; ++j) {
          var range = intact[j];
          if (change.to <= range.from)
            intact2.push({from: range.from + diff, to: range.to + diff, domStart: range.domStart});
          else if (range.to <= change.from)
            intact2.push(range);
          else {
            if (change.from > range.from)
              intact2.push({from: range.from, to: change.from, domStart: range.domStart})
            if (change.to < range.to)
              intact2.push({from: change.to + diff, to: range.to + diff,
                            domStart: range.domStart + (change.to - range.from)});
          }
        }
        intact = intact2;
      }

      var from = Math.min(showingFrom, Math.max(visibleFrom - 3, 0)),
          to = Math.min(lines.length, Math.max(showingTo, visibleTo + 3)),
          updates = [], domPos = 0, domEnd = showingTo - showingFrom, pos = from, changedLines = 0;

      for (var i = 0, l = intact.length; i < l; ++i) {
        var range = intact[i];
        if (range.to <= from) continue;
        if (range.from >= to) break;
        if (range.domStart > domPos || range.from > pos) {
          updates.push({from: pos, to: range.from, domSize: range.domStart - domPos, domStart: domPos});
          changedLines += range.from - pos;
        }
        pos = range.to;
        domPos = range.domStart + (range.to - range.from);
      }
      if (domPos != domEnd || pos != to) {
        changedLines += Math.abs(to - pos);
        updates.push({from: pos, to: to, domSize: domEnd - domPos, domStart: domPos});
      }

      if (!updates.length) return;
      lineDiv.style.display = "none";
      // TODO heuristic, do some tests
      if (changedLines > (visibleTo - visibleFrom) * .3)
        refreshDisplay(from = Math.max(visibleFrom - 10, 0), to = Math.min(visibleTo + 7, lines.length));
      else
        patchDisplay(updates);
      lineDiv.style.display = "";

      var different = from != showingFrom || to != showingTo;
      showingFrom = from; showingTo = to;
      mover.style.top = (from * lineHeight()) + "px";
      if (different) {
        code.style.height = (lines.length * lineHeight() + 2 * paddingTop()) + "px";
        updateGutter();
      }

      if (lineDiv.childNodes.length != showingTo - showingFrom)
        throw new Error("BAD PATCH! " + JSON.stringify(updates) + " size=" + (showingTo - showingFrom) +
                        " nodes=" + lineDiv.childNodes.length);
    }

    function refreshDisplay(from, to) {
      var html = [], start = {line: from, ch: 0}, inSel = posLess(sel.from, start) && !posLess(sel.to, start);
      for (var i = from; i < to; ++i) {
        var ch1 = null, ch2 = null;
        if (inSel) {
          ch1 = 0;
          if (sel.to.line == i) {inSel = false; ch2 = sel.to.ch;}
        }
        else if (sel.from.line == i) {
          if (sel.to.line == i) {ch1 = sel.from.ch; ch2 = sel.to.ch;}
          else {inSel = true; ch1 = sel.from.ch;}
        }
        html.push(lines[i].getHTML(ch1, ch2, true));
      }
      lineDiv.innerHTML = html.join("");
    }
    function patchDisplay(updates) {
      // Slightly different algorithm for IE, since there .innerHTML on PRE nodes is dumb.
      var sfrom = sel.from.line, sto = sel.to.line, off = 0,
          scratch = badInnerHTML && document.createElement("div");
      for (var i = 0, e = updates.length; i < e; ++i) {
        var rec = updates[i];
        var extra = (rec.to - rec.from) - rec.domSize;
        var nodeAfter = lineDiv.childNodes[rec.domStart + rec.domSize + off] || null;
        if (badInnerHTML)
          for (var j = Math.max(-extra, rec.domSize); j > 0; --j)
            lineDiv.removeChild(nodeAfter ? nodeAfter.previousSibling : lineDiv.lastChild);
        else if (extra) {
          for (var j = Math.max(0, extra); j > 0; --j)
            lineDiv.insertBefore(document.createElement("pre"), nodeAfter);
          for (var j = Math.max(0, -extra); j > 0; --j)
            lineDiv.removeChild(nodeAfter ? nodeAfter.previousSibling : lineDiv.lastChild);
        }
        var node = lineDiv.childNodes[rec.domStart + off], inSel = sfrom < rec.from && sto >= rec.from;
        for (var j = rec.from; j < rec.to; ++j) {
          var ch1 = null, ch2 = null;
          if (inSel) {
            ch1 = 0;
            if (sto == j) {inSel = false; ch2 = sel.to.ch;}
          }
          else if (sfrom == j) {
            if (sto == j) {ch1 = sel.from.ch; ch2 = sel.to.ch;}
            else {inSel = true; ch1 = sel.from.ch;}
          }
          if (badInnerHTML) {
            scratch.innerHTML = lines[j].getHTML(ch1, ch2, true);
            lineDiv.insertBefore(scratch.firstChild, nodeAfter);
          }
          else {
            node.innerHTML = lines[j].getHTML(ch1, ch2, false);
            node = node.nextSibling;
          }
        }
        off += extra;
      }
    }
    function updateGutter() {
      if (!options.gutter) return;
      gutter.style.height = Math.max(lineDiv.offsetHeight, div.clientHeight) + "px";
      var html = [];
      for (var i = showingFrom; i < showingTo; ++i) {
        var marker = lines[i].gutterMarker;
        var text = (marker && marker.text && htmlEscape(marker.text)) ||
                   (options.lineNumbers && i + options.firstLineNumber) || "\u00a0";
        html.push((marker && marker.style ? '<pre class="' + marker.style + '">' : "<pre>"), text, "</pre>");
      }
      gutter.style.display = "none";
      gutter.innerHTML = html.join("");
      var minwidth = String(lines.length).length, firstNode = gutter.firstChild.firstChild, val = firstNode.nodeValue;
      while (val.length < minwidth) val = "\u00a0" + val;
      firstNode.nodeValue = val;
      gutter.style.display = "";
      lineWrap.style.marginLeft = gutter.offsetWidth + "px";
    }

    function setSelection(from, to, oldFrom, oldTo) {
      if (posEq(sel.from, from) && posEq(sel.to, to)) return;
      var sh = shiftSelecting && clipPos(shiftSelecting);
      if (posLess(to, from)) {var tmp = to; to = from; from = tmp;}
      if (sh) {
        if (posLess(sh, from)) from = sh;
        else if (posLess(to, sh)) to = sh;
      }

      var startEq = posEq(sel.to, to), endEq = posEq(sel.from, from);
      if (posEq(from, to)) sel.inverted = false;
      else if (startEq && !endEq) sel.inverted = true;
      else if (endEq && !startEq) sel.inverted = false;

      if (oldFrom == null) {oldFrom = sel.from.line; oldTo = sel.to.line;}
      if (posEq(from, to)) {
        if (!posEq(sel.from, sel.to))
          changes.push({from: oldFrom, to: oldTo + 1});
      }
      else if (posEq(sel.from, sel.to)) {
        changes.push({from: from.line, to: to.line + 1});
      }
      else {
        if (!posEq(from, sel.from)) {
          if (from.line < oldFrom)
            changes.push({from: from.line, to: Math.min(to.line, oldFrom) + 1});
          else
            changes.push({from: oldFrom, to: Math.min(oldTo, from.line) + 1});
        }
        if (!posEq(to, sel.to)) {
          if (to.line < oldTo)
            changes.push({from: Math.max(oldFrom, from.line), to: oldTo + 1});
          else
            changes.push({from: Math.max(from.line, oldTo), to: to.line + 1});
        }
      }
      sel.from = from; sel.to = to;
      selectionChanged = true;
    }

    function updateCursor() {
      var head = sel.inverted ? sel.from : sel.to;
      var x = charX(head.line, head.ch) + "px", y = (head.line - showingFrom) * lineHeight() + "px";
      inputDiv.style.top = y; inputDiv.style.left = x;
      if (posEq(sel.from, sel.to)) {
        cursor.style.top = y; cursor.style.left = x;
        cursor.style.display = "";
      }
      else cursor.style.display = "none";
    }
    function clipLine(n) {return Math.max(0, Math.min(n, lines.length-1));}
    function clipPos(pos) {
      if (pos.line < 0) return {line: 0, ch: 0};
      if (pos.line >= lines.length) return {line: lines.length-1, ch: lines[lines.length-1].text.length};
      var ch = pos.ch;
      if (ch < 0) ch = 0;
      else if (ch > lines[pos.line].text.length) ch = lines[pos.line].text.length;
      return ch == pos.ch ? pos : {line: pos.line, ch: ch};
    }
    function setCursor(line, ch) {
      var pos = clipPos({line: line, ch: ch || 0});
      setSelection(pos, pos);
    }
    function scrollPage(down) {
      var linesPerPage = Math.floor(div.clientHeight / lineHeight()), head = sel.inverted ? sel.from : sel.to;
      setCursor(head.line + (Math.max(linesPerPage - 1, 1) * (down ? 1 : -1)), head.ch);
    }
    function scrollEnd(top) {
      setCursor(top ? 0 : lines.length - 1);
    }
    function selectAll() {
      var endLine = lines.length - 1;
      setSelection({line: 0, ch: 0}, {line: endLine, ch: lines[endLine].text.length});
    }
    function selectWordAt(pos) {
      var line = lines[pos.line].text;
      var start = pos.ch, end = pos.ch;
      while (start > 0 && /\w/.test(line.charAt(start - 1))) --start;
      while (end < line.length - 1 && /\w/.test(line.charAt(end))) ++end;
      setSelection({line: pos.line, ch: start}, {line: pos.line, ch: end});
    }
    function handleEnter() {
      replaceSelection("\n", "end");
      if (options.enterMode != "flat")
        indentLine(sel.from.line, options.enterMode == "keep" ? "prev" : "smart");
    }
    function handleTab(shift) {
      shiftSelecting = null;
      switch (options.tabMode) {
      case "default":
        return false;
      case "indent":
        for (var i = sel.from.line, e = sel.to.line; i <= e; ++i) indentLine(i, "smart");
        break;
      case "classic":
        if (posEq(sel.from, sel.to)) {replaceSelection("\t", "end"); break;}
      case "shift":
        for (var i = sel.from.line, e = sel.to.line; i <= e; ++i) indentLine(i, shift ? "subtract" : "add");
        break;
      }
      return true;
    }
    function indentLine(n, how) {
      if (how == "smart") {
        if (!mode.indent) how = "prev";
        else {
          var state = getStateBefore(n);
          if (!state) how = "prev";
        }
      }

      var line = lines[n], curSpace = line.indentation(), curSpaceString = line.text.match(/^\s*/)[0], indentation;
      if (how == "prev") {
        if (n) indentation = lines[n-1].indentation();
        else indentation = 0;
      }
      else if (how == "smart") indentation = mode.indent(state, line.text.slice(curSpace));
      else if (how == "add") indentation = curSpace + options.indentUnit;
      else if (how == "subtract") indentation = Math.max(0, curSpace - options.indentUnit);
      var diff = indentation - curSpace;

      if (!diff) {
        if (sel.from.line != n && sel.to.line != n) return;
        var indentString = curSpaceString;
      }
      else {
        var indentString = "", pos = 0;
        if (options.indentWithTabs)
          for (var i = Math.floor(indentation / tabSize); i; --i) {pos += tabSize; indentString += "\t";}
        while (pos < indentation) {++pos; indentString += " ";}
      }

      replaceRange(indentString, {line: n, ch: 0}, {line: n, ch: curSpaceString.length});
    }

    function replaceRange(code, from, to) {
      from = clipPos(from);
      if (!to) to = from; else to = clipPos(to);
      code = splitLines(code);
      function adjustPos(pos) {
        if (posLess(pos, from)) return pos;
        if (posLess(pos, to)) return end;
        var line = pos.line + code.length - (to.line - from.line) - 1;
        var ch = pos.ch;
        if (pos.line == to.line)
          ch += code[0].length - (to.ch - (to.line == from.line ? from.ch : 0));
        return {line: line, ch: ch};
      }
      var end;
      replaceRange1(code, from, to, function(end1) {
        end = end1;
        return {from: adjustPos(sel.from), to: adjustPos(sel.to)};
      });
      return end;
    }
    function replaceRange1(code, from, to, computeSel) {
      var endch = code.length == 1 ? code[0].length + from.ch : code[code.length-1].length;
      var newSel = computeSel({line: from.line + code.length - 1, ch: endch});
      updateLines(from, to, code, newSel.from, newSel.to);
    }

    function replaceSelection(code, collapse) {
      replaceRange1(splitLines(code), sel.from, sel.to, function(end) {
        if (collapse == "end") return {from: end, to: end};
        else if (collapse == "start") return {from: sel.from, to: sel.from};
        else return {from: sel.from, to: end};
      });
    }
    function getSelection(lineSep) {
      var l1 = sel.from.line, l2 = sel.to.line;
      if (l1 == l2) return lines[l1].text.slice(sel.from.ch, sel.to.ch);
      var code = [lines[l1].text.slice(sel.from.ch)];
      for (var i = l1 + 1; i < l2; ++i) code.push(lines[i].text);
      code.push(lines[l2].text.slice(0, sel.to.ch));
      return code.join(lineSep || "\n");
    }

    function setMode(spec) {
      options.mode = spec;
      mode = CodeMirror.getMode(options, spec);
      for (var i = 0, l = lines.length; i < l; ++i)
        lines[i].stateAfter = null;
      work = [0];
    }
    function setLineNumbers(on) {
      options.lineNumbers = on;
      if (on && !options.gutter) options.gutter = "ln";
      else if (!on && options.gutter == "ln") options.gutter = false;
      gutterChanged();
    }
    function setGutter(on) {
      options.gutter = !on && options.lineNumbers ? "ln" : !!on;
      gutterChanged();
    }
    function gutterChanged() {
      gutter.style.display = options.gutter ? "" : "none";
      if (options.gutter) updateGutter();
      else lineDiv.parentNode.style.marginLeft = 0;
    }

    function addGutterMarker(line, text, className) {
      if (typeof line == "number") line = lines[clipLine(line)];
      line.gutterMarker = {text: text, style: className};
      updateGutter();
      return line;
    }
    function removeGutterMarker(line) {
      if (typeof line == "number") line = lines[clipLine(line)];
      line.gutterMarker = null;
      updateGutter();
    }

    function charX(line, pos) {
      var text = lines[line].text;
      if (text.lastIndexOf("\t", pos) == -1) return pos * charWidth();
      try {
        measure.firstChild.firstChild.nodeValue = text.slice(0, pos);
        return measure.firstChild.offsetWidth;
      } finally {measure.firstChild.firstChild.nodeValue = "-";}
    }
    function charFromX(line, x) {
      var text = lines[line].text, cw = charWidth();
      if (text.indexOf("\t") == -1) return Math.min(text.length, Math.round(x / cw));
      var mspan = measure.firstChild, mtext = mspan.firstChild;
      try {
        var from = 0, to = text.length;
        for (;;) {
          if (to - from <= 1) return from;
          var middle = Math.ceil((from + to) / 2);
          mtext.nodeValue = text.slice(0, middle);
          if (mspan.offsetWidth > x) to = middle;
          else from = middle;
        }
      } finally {mtext.nodeValue = "-";}
    }
    function localCursorCoords(start) {
      var head = start ? sel.from : sel.to;
      return {x: charX(head.line, head.ch), y: head.line * lineHeight()};
    }
    function cursorCoords(start) {
      var local = localCursorCoords(start), off = eltOffset(space);
      return {x: off.left + local.x, y: off.top + local.y, yBot: off.top + local.y + lineHeight()};
    }

    function lineHeight() {
      var nlines = lineDiv.childNodes.length;
      if (nlines) return lineDiv.offsetHeight / nlines;
      else return measure.offsetHeight || 1;
    }
    function charWidth() {return measure.firstChild.offsetWidth || 1;}
    function paddingTop() {return lineWrap.offsetTop;}
    function paddingLeft() {return lineWrap.offsetLeft;}

    function posFromMouse(e) {
      var off = eltOffset(lineWrap),
          x = e.pageX() - off.left,
          y = e.pageY() - off.top;
      if (e.target() == code && y < (lines.length * lineHeight())) return null;
      var line = showingFrom + Math.floor(y / lineHeight());
      return clipPos({line: line, ch: charFromX(clipLine(line), x)});
    }

    function restartBlink() {
      clearInterval(blinker);
      var on = true;
      cursor.style.visibility = "";
      blinker = setInterval(function() {
        cursor.style.visibility = (on = !on) ? "" : "hidden";
      }, 650);
    }
    function markText(from, to, className) {
      from = clipPos(from); to = clipPos(to);
      var accum = [];
      function add(line, from, to, className) {
        var line = lines[line], mark = line.addMark(from, to, className);
        mark.line = line;
        accum.push(mark);
      }
      if (from.line == to.line) add(from.line, from.ch, to.ch, className);
      else {
        add(from.line, from.ch, null, className);
        for (var i = from.line + 1, e = to.line; i < e; ++i)
          add(i, 0, null, className);
        add(to.line, 0, to.ch, className);
      }
      changes.push({from: from.line, to: to.line + 1});
      return function() {
        var start, end;
        for (var i = 0; i < accum.length; ++i) {
          var mark = accum[i], found = indexOf(lines, mark.line);
          mark.line.removeMark(mark);
          if (found > -1) {
            if (start == null) start = found;
            end = found;
          }
        }
        if (start != null) changes.push({from: start, to: end + 1});
      };
    }

    var matching = {"(": ")>", ")": "(<", "[": "]>", "]": "[<", "{": "}>", "}": "{<"};
    function matchBrackets(autoclear) {
      var head = sel.inverted ? sel.from : sel.to, line = lines[head.line], pos = head.ch - 1;
      var match = (pos >= 0 && matching[line.text.charAt(pos)]) || matching[line.text.charAt(++pos)];
      if (!match) return;
      var ch = match.charAt(0), forward = match.charAt(1) == ">", d = forward ? 1 : -1, st = line.styles;
      for (var off = forward ? pos + 1 : pos, i = 0, e = st.length; i < st; i+=2)
        if ((off -= st[i].length) <= 0) {var style = st[i+1]; break;}

      var stack = [line.text.charAt(pos)], re = /[(){}[\]]/;
      function scan(line, from, to) {
        if (!line.text) return;
        var st = line.styles, pos = forward ? 0 : line.text.length - 1, cur;
        for (var i = forward ? 0 : st.length - 2, e = forward ? st.length : -2; i != e; i += 2*d) {
          var text = st[i];
          if (st[i+1] != null && st[i+1] != style) {pos += d * text.length; continue;}
          for (var j = forward ? 0 : text.length - 1, te = forward ? text.length : -1; j != te; j += d, pos+=d) {
            if (pos >= from && pos < to && re.test(cur = text.charAt(j))) {
              var match = matching[cur];
              if (match.charAt(1) == ">" == forward) stack.push(cur);
              else if (stack.pop() != match.charAt(0)) return {pos: pos, match: false};
              else if (!stack.length) return {pos: pos, match: true};
            }
          }
        }
      }
      for (var i = head.line, e = forward ? Math.min(i + 30, lines.length) : Math.max(0, i - 30); i != e; i+=d) {
        var line = lines[i], first = i == head.line;
        var found = scan(line, first && forward ? pos + 1 : 0, first && !forward ? pos : line.text.length);
        if (found) {
          var style = found.match ? "CodeMirror-matchingbracket" : "CodeMirror-nonmatchingbracket";
          var one = markText({line: head.line, ch: pos}, {line: head.line, ch: pos+1}, style),
              two = markText({line: i, ch: found.pos}, {line: i, ch: found.pos + 1}, style);
          var clear = operation(function(){one(); two();});
          if (autoclear) setTimeout(clear, 800);
          else bracketHighlighted = clear;
          break;
        }
      }
    }

    function getStateBefore(n) {
      var state;
      for (var search = n - 1, lim = n - 40;; --search) {
        if (search < 0) {state = mode.startState ? mode.startState() : true; break;}
        if (search < lim) return null;
        if (state = lines[search].stateAfter) {state = copyState(mode, state); break;}
      }
      for (++search; search < n; ++search) {
        var line = lines[search];
        line.highlight(mode, state);
        line.stateAfter = copyState(mode, state);
      }
      if (!lines[n].stateAfter) work.push(n);
      return state;
    }
    function highlightWorker(start) {
      var end = +new Date + options.workTime;
      while (work.length) {
        if (!lines[showingFrom].stateAfter) var task = showingFrom;
        else var task = work.pop()
        if (task >= lines.length || lines[task].stateAfter) continue;
        var minindent = lines[task].indentation(), minline = task;
        for (var e = Math.max(1, task - 50); task >= e; --task) {
          var line = lines[task-1], state = line.stateAfter, indent = line.indentation();
          if (state) break;
          if (indent < minindent) {minindent = indent; minline = task;}
        }
        if (state) state = copyState(mode, state);
        else {
          state = mode.startState ? mode.startState() : true;
          if (task) task = minline;
        }

        for (var i = task, l = lines.length; i < l; ++i) {
          var line = lines[i];
          // TODO don't bail out so quickly -- changes in previous lines might require a change here
          if (line.stateAfter) break;
          if (+new Date > end) {
            work.push(i);
            startWorker(options.workDelay);
            changes.push({from: task, to: i});
            return;
          }
          line.highlight(mode, state);
          line.stateAfter = copyState(mode, state);
        }
        changes.push({from: task, to: i});
      }
    }
    function startWorker(time) {
      if (!work.length) return;
      highlight.set(time, operation(highlightWorker));
    }

    function startOperation() {
      updateInput = null; changes = []; textChanged = false; selectionChanged = false;
    }
    function endOperation() {
      if (selectionChanged) scrollCursorIntoView();
      if (changes.length) updateDisplay(changes);

      if (selectionChanged) {updateCursor(); restartBlink();}
      if (updateInput === true || (updateInput !== false && selectionChanged))
        prepareInput();

      if (selectionChanged && options.onCursorActivity)
        options.onCursorActivity(instance);
      if (textChanged && options.onChange)
        options.onChange(instance);
      if (selectionChanged && options.matchBrackets)
        setTimeout(operation(function() {
          if (bracketHighlighted) {bracketHighlighted(); bracketHighlighted = null;}
          matchBrackets(false);
        }), 20);
    }
    var nestedOperation = 0;
    function operation(f) {
      return function() {
        if (!nestedOperation++) startOperation();
        try {var result = f.apply(this, arguments);}
        finally {if (!--nestedOperation) endOperation();}
        return result;
      };
    }

    function SearchCursor(query, pos, caseFold) {
      this.atOccurrence = false;
      if (caseFold == undefined) caseFold = typeof query == "string" && query == query.toLowerCase();

      if (pos && typeof pos == "object") pos = clipPos(pos);
      else pos = {line: 0, ch: 0};
      this.pos = {from: pos, to: pos};

      if (typeof query != "string") // Regexp match
        this.matches = function(reverse, pos) {
          if (reverse) {
            var line = lines[pos.line].text.slice(0, pos.ch), match = line.match(query), start = 0;
            while (match) {
              var ind = line.indexOf(match[0]);
              start += ind;
              line = line.slice(ind + 1);
              var newmatch = line.match(query);
              if (newmatch) match = newmatch;
              else break;
            }
          }
          else {
            var line = lines[pos.line].text.slice(pos.ch), match = line.match(query),
                start = match && pos.ch + line.indexOf(match[0]);
          }
          if (match)
            return {from: {line: pos.line, ch: start},
                    to: {line: pos.line, ch: start + match[0].length},
                    match: match};
        };
      else { // String match
        if (caseFold) query = query.toLowerCase();
        var fold = caseFold ? function(str){return str.toLowerCase();} : function(str){return str;};
        var target = query.split("\n");
        if (target.length == 1)
          this.matches = function(reverse, pos) {
            var line = fold(lines[pos.line].text), len = query.length, match;
            if (reverse ? (pos.ch >= len && (match = line.lastIndexOf(query, pos.ch - len)) != -1)
                        : (match = line.indexOf(query, pos.ch)) != -1)
              return {from: {line: pos.line, ch: match},
                      to: {line: pos.line, ch: match + len}};
          };
        else
          this.matches = function(reverse, pos) {
            var ln = pos.line, idx = (reverse ? target.length - 1 : 0), match = target[idx], line = fold(lines[ln].text);
            var offsetA = (reverse ? line.indexOf(match) + match.length : line.lastIndexOf(match));
            if (reverse ? offsetA >= pos.ch || offsetA != match.length
                        : offsetA <= pos.ch || offsetA != line.length - match.length)
              return;
            while (true) {
              if (reverse ? !ln : ln == lines.length - 1) return;
              line = fold(lines[ln += reverse ? -1 : 1].text);
              match = target[reverse ? --idx : ++idx];
              if (idx > 0 && idx < target.length - 1) {
                if (line != match) return;
                else continue;
              }
              var offsetB = (reverse ? line.lastIndexOf(match) : line.indexOf(match) + match.length);
              if (reverse ? offsetB != line.length - match.length : offsetB != match.length)
                return;
              var start = {line: pos.line, ch: offsetA}, end = {line: ln, ch: offsetB};
              return {from: reverse ? end : start, to: reverse ? start : end};
            }
          };
      }
    }

    SearchCursor.prototype = {
      findNext: function() {return this.find(false);},
      findPrevious: function() {return this.find(true);},

      find: function(reverse) {
        var self = this, pos = clipPos(reverse ? this.pos.from : this.pos.to);
        function savePosAndFail(line) {
          var pos = {line: line, ch: 0};
          self.pos = {from: pos, to: pos};
          self.atOccurrence = false;
          return false;
        }

        while (true) {
          if (this.pos = this.matches(reverse, pos)) {
            this.atOccurrence = true;
            return this.pos.match || true;
          }

          if (reverse) {
            if (!pos.line) return savePosAndFail(0);
            pos = {line: pos.line-1, ch: lines[pos.line-1].text.length};
          }
          else {
            if (pos.line == lines.length - 1) return savePosAndFail(lines.length);
            pos = {line: pos.line+1, ch: 0};
          }
        }
      },

      from: function() {if (this.atOccurrence) return copyPos(this.pos.from);},
      to: function() {if (this.atOccurrence) return copyPos(this.pos.to);}
    };

    function isLine(l) {return l >= 0 && l < lines.length;}

    var instance = {
      getValue: getValue,
      setValue: operation(setValue),
      getSelection: getSelection,
      replaceSelection: operation(replaceSelection),
      focus: function(){input.focus(); onFocus();},
      setOption: function(option, value) {
        if (option == "mode") setMode(value);
        else if (option === "lineNumbers") setLineNumbers(value);
        else if (option === "gutter") setGutter(value);
        else if (option === "indentUnit") {options.indentUnit = value; setMode(options.mode);}
        else options[option] = value;
      },
      cursorCoords: cursorCoords,
      undo: operation(undo),
      redo: operation(redo),
      getSearchCursor: function(query, pos, caseFold) {return new SearchCursor(query, pos, caseFold);},
      markText: operation(function(a, b, c){return operation(markText(a, b, c));}),
      setMarker: addGutterMarker,
      clearMarker: removeGutterMarker,
      matchBrackets: operation(function(){matchBrackets(true);}),

      lineCount: function() {return lines.length;},
      getCursor: function(start) {return copyPos(start ? sel.from : sel.to);},
      setCursor: operation(function(line, ch) {
        if (ch == null && typeof line.line == "number") setCursor(line.line, line.ch);
        else setCursor(line, ch);
      }),
      setSelection: operation(function(start, end) {setSelection(clipPos(start), end ? clipPos(end) : start);}),
      getLine: function(line) {if (isLine(line)) return lines[line].text;},
      setLine: operation(function(line, text) {
        if (isLine(line)) replaceRange(text, {line: line, ch: 0}, {line: line, ch: lines[line].text.length});
      }),
      removeLine: operation(function(line) {
        if (isLine(line)) replaceRange("", {line: line, ch: 0}, clipPos({line: line+1, ch: 0}));
      }),
      replaceRange: operation(replaceRange),

      operation: function(f){return operation(f)();},
      refresh: function(){updateDisplay([{from: 0, to: lines.length}]);},
      getInputField: function(){return input;}
    };
    return instance;
  }

  CodeMirror.defaults = {
    value: "",
    mode: null,
    indentUnit: 2,
    indentWithTabs: false,
    tabMode: "classic",
    enterMode: "indent",
    lineNumbers: false,
    gutter: false,
    firstLineNumber: 1,
    readOnly: false,
    onChange: null,
    onCursorActivity: null,
    onGutterClick: null,
    matchBrackets: false,
    workTime: 200,
    workDelay: 300,
    undoDepth: 40,
    tabindex: null
  };

  var modes = {}, mimeModes = {};
  CodeMirror.defineMode = function(name, mode) {
    if (!CodeMirror.defaults.mode) CodeMirror.defaults.mode = name;
    modes[name] = mode;
  };
  CodeMirror.defineMIME = function(mime, spec) {
    mimeModes[mime] = spec;
  };
  CodeMirror.getMode = function(options, spec) {
    if (typeof spec == "string" && /^\w+\/\w+$/.test(spec)) {
      if (!mimeModes.hasOwnProperty(spec)) throw new Error("No mode defined for MIME " + spec);
      spec = mimeModes[spec];
    }
    if (typeof spec == "string")
      var mname = spec, config = {};
    else
      var mname = spec.name, config = spec;
    var mfactory = modes[mname];
    if (!mfactory) throw new Error("No mode " + mname + " found.");
    return mfactory(options, config);
  }

  CodeMirror.fromTextArea = function(textarea, options) {
    if (!options) options = {};
    options.value = textarea.value;
    if (!options.tabindex && textarea.tabindex)
      options.tabindex = textarea.tabindex;

    function save() {textarea.value = instance.getValue();}
    if (textarea.form) {
      var rmSubmit = connect(textarea.form, "submit", save);
      var realSubmit = textarea.form.submit;
      function wrappedSubmit() {
        updateField();
        textarea.form.submit = realSubmit;
        textarea.form.submit();
        textarea.form.submit = wrappedSubmit;
      }
      textarea.form.submit = wrappedSubmit;
    }

    textarea.style.display = "none";
    var instance = CodeMirror(function(node) {
      textarea.parentNode.insertBefore(node, textarea.nextSibling);
    }, options);
    instance.save = save;
    instance.toTextArea = function() {
      save();
      textarea.parentNode.removeChild(instance.div);
      textarea.style.display = "";
      if (textarea.form) {
        textarea.form.submit = realSubmit;
        rmSubmit();
      }
    };
    return instance;
  };

  function StringStream(string) {
    this.pos = this.start = 0;
    this.string = string;
  }
  StringStream.prototype = {
    eol: function() {return this.pos >= this.string.length;},
    peek: function() {return this.string.charAt(this.pos);},
    next: function() {
      if (this.pos < this.string.length)
        return this.string.charAt(this.pos++);
    },
    eat: function(match) {
      var ch = this.string.charAt(this.pos);
      if (typeof match == "string") var ok = ch == match;
      else var ok = ch && (match.test ? match.test(ch) : match(ch));
      if (ok) {++this.pos; return ch;}
    },
    eatWhile: function(match) {
      var start = this.start;
      while (this.eat(match)){}
      return this.pos > start;
    },
    eatSpace: function() {
      var start = this.pos;
      while (/[\s\u00a0]/.test(this.string.charAt(this.pos))) ++this.pos;
      return start < this.pos;
    },
    backUp: function(n) {this.pos -= n;},
    column: function() {return this.start;},
    indentation: function() {return countIndentation(this.string);},
    match: function(pattern, consume, caseInsensitive) {
      if (typeof pattern == "string") {
        function cased(str) {return caseInsensitive ? str.toLowerCase() : str;}
        if (cased(this.string).indexOf(cased(pattern), this.pos) == this.pos) {
          if (consume !== false) this.pos += pattern.length;
          return true;
        }
      }
      else {
        var match = this.string.slice(this.pos).match(pattern);
        if (match && consume !== false) this.pos += match[0].length;
        return match;
      }
    },
    current: function(){return this.string.slice(this.start, this.pos);}
  };

  function copyState(mode, state) {
    if (state === true) return state;
    if (mode.copyState) return mode.copyState(state);
    var nstate = {};
    for (var n in state) {
      var val = state[n];
      if (val instanceof Array) val = val.concat([]);
      nstate[n] = val;
    }
    return nstate;
  }
  CodeMirror.copyState = copyState;

  return CodeMirror;
})();
